/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.compilation.utils.popup.actions;

import java.io.File;
import java.util.LinkedList;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import fr.irisa.espresso.sme.RootModel;
import fr.irisa.espresso.sme.compilation.Scenario;
import fr.irisa.espresso.sme.compilation.utils.dialogs.FindCompilationScenarioDialog;
import fr.irisa.espresso.sme.compilation.utils.jobs.CompilingMethods;
import fr.irisa.espresso.sme.compilation.utils.jobs.ExecuteScenarioJob;
import fr.irisa.espresso.sme.polychrony.EnvironmentConstants;
import fr.irisa.espresso.sme.polychrony.ExitStatus;
import fr.irisa.espresso.sme.polychrony.GraphValue;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyConsole;

public class GenerateAction implements IObjectActionDelegate {
	private static final String	SCENARIO_EXECUTION		= "Execute a compilation scenario";
	private static final String	C_GENERATION			= "Generate C files";
	private static final String	CPP_GENERATION			= "Generate CPP files";
	private static final String	JAVA_GENERATION			= "Generate Java files";
	private static final String	SIG_GENERATION			= "Generate Signal textual file";
	private static final String	SIG_LIS_GENERATION		= "Generate Signal textual (LIS) file";
	private static final String	SIG_TRA_GENERATION		= "Generate Signal textual (TRA) file";
	private static final String	SIGNAL_MODEL_GENERATION	= "Generate Signal model file";
	private static final String	GENERAL_COMPILATION		= "General Compilation";
	private static final String	INTERACTIVE_COMPILATION	= "Interactive Compilation";
	
	/** The file on which the transformations will be applied */
	private IFile				sourceFile;
	
	public GenerateAction() {
		super();
	}
	
	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {}
	
	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		
		CompilationUtils compilationUtils = new CompilationUtils();
		String actionID = action.getText();
		compilationUtils.setResources(new LinkedList<IResource>());
		if (sourceFile != null) {
			// the directory of the file is added to the list of resources to be refreshed
			compilationUtils.getResources().add(sourceFile.getParent());
			
			// case of the scenario execution (the .ssc file written in the Eclipse "SME Scenario" tab)
			if (actionID.equals(SCENARIO_EXECUTION)) {
				FindCompilationScenarioDialog dialog = new FindCompilationScenarioDialog(new Shell());
				if (dialog.open() == Window.OK) {
					Scenario scenario = dialog.getScenario();
					if (scenario != null) {
						executeScenario(scenario, compilationUtils);
					}
					else {
						MessageDialog.openError(new Shell(), CompilationUtils.ERROR_SCENARIO,
							CompilationUtils.ERROR_NO_SCENARIO_SELECTED);
					}
				}
			}
			else {
				String projectPath = compilationUtils.getPluginPath("fr.irisa.espresso.sme.compilation.utils");
				projectPath += GenerateAction.class.getResource("").getPath();
				boolean isSimpleScenario = true;
				
				File file = null;
				if (actionID.equals(C_GENERATION)) {
					file = new File(projectPath + "CGeneration.ssc");
				}
				else if (actionID.equals(CPP_GENERATION)) {
					file = new File(projectPath + "CPPGeneration.ssc");
				}
				else if (actionID.equals(JAVA_GENERATION)) {
					file = new File(projectPath + "JavaGeneration.ssc");
				}
				else if (actionID.equals(SIG_GENERATION)) {
					file = new File(projectPath + "SIGGeneration.ssc");
				}
				else if (actionID.equals(SIG_LIS_GENERATION)) {
					file = new File(projectPath + "SIGLISGeneration.ssc");
				}
				else if (actionID.equals(SIG_TRA_GENERATION)) {
					file = new File(projectPath + "SIGTRAGeneration.ssc");
				}
				else if (actionID.equals(SIGNAL_MODEL_GENERATION)) {
					file = new File(projectPath + "SMEGeneration.ssc");
				}
				else {
					isSimpleScenario = false;
				}
				
				// case of a simple scenario (C, C++...)
				if (isSimpleScenario == true) {
					Scenario scenario = compilationUtils.checkFile(file);
					executeScenario(scenario, compilationUtils);
				}
				// interactive and general compilation
				else if ((actionID.equals(INTERACTIVE_COMPILATION)) || (actionID.equals(GENERAL_COMPILATION))) {
					CompilingMethods compilingMethods = new CompilingMethods(compilationUtils);
					long tree = 0;
					boolean isSignalTextualFile = (sourceFile.getFileExtension()
							.equalsIgnoreCase(CompilationUtils.EXTENSION_SIGNAL_TEXTUAL_FILE));
					boolean isSignalEMFModel = (sourceFile.getFileExtension()
							.equalsIgnoreCase(CompilationUtils.EXTENSION_SIGNAL_EMF_MODEL));
					
					compilationUtils.printBanner();
					PolychronyConsole.printInfo("Input file : " + sourceFile.getFullPath() + "\n\n");
					
					// reset of the string memory
					PkPlugin.getServices().resetMachine();
					compilationUtils.setSaveLibraryPath(PkPlugin.getServices()
							.getEnv(EnvironmentConstants.LIBRARY_PATH));
					
					IPath fileLocation = sourceFile.getLocation();
					IPath fileDirectory = fileLocation.removeFileExtension();
					if (isSignalTextualFile == true) {
						tree = PkPlugin.getParser().parseSrcfile(fileLocation.toOSString());
					}
					else if (isSignalEMFModel == true) {
						ResourceSet resourceSet = new ResourceSetImpl();
						URI fileURI = URI.createFileURI(fileLocation.toOSString());
						Resource resource = resourceSet.getResource(fileURI, true);
						if (resource.getContents().size() == 1) {
							EObject obj = resource.getContents().get(0);
							if (obj instanceof RootModel) // The root of the EMF model
							{
								RootModel rootModel = (RootModel) obj;
								if (compilationUtils.validModel(resource)) {
									compilingMethods.generateSignalModules(obj, sourceFile);
									// the SME/Signal interface hashmap is cleared before every new execution
									PkPlugin.getTraceabilityAPI().clearMap();
									tree = rootModel.getDeclarationAST(PKTrees.ERROR_NODE);
									PkPlugin.getTraceabilityAPI().initializeSMENodesList();
									
									/*
									 * Information printed in the console if the booleans are true. Please refer to the
									 * documentation of these methods.
									 */
									PkPlugin.getTraceabilityAPI().printInformations(false, false);
									PkPlugin.getTraceabilityAPI().printTree(tree, 0, false);
								}
								else {
									PolychronyConsole.printError("Error detected : "
											+ CompilationUtils.ERROR_INVALID_MODEL);
								}
							}
							else {
								PolychronyConsole
										.printError("Error detected : " + CompilationUtils.ERROR_NO_ROOT_MODEL);
							}
						}
						else {
							PolychronyConsole.printError("Error detected : "
									+ CompilationUtils.ERROR_MORE_THAN_ONE_ROOT_ELEMENT);
						}
					}
					
					// Generation of SIGNAL form everytime
					compilingMethods.generateSignal(tree, fileDirectory);
					
					if (tree != PKTrees.ERROR_NODE) {
						compilationUtils.addPathSignalLibraryPath(fileDirectory);
						
						// Tree to Graph transformation
						GraphValue graphValue = PkPlugin.getServices().tree2DCG(tree, true,
							PkPlugin.getTreeAPI().getNilTree(), true, fileDirectory.toOSString(), false);
						
						// If there is no problem in the graph transformation, for the interactive compilation
						if ((graphValue.getReturnCode() == ExitStatus.NO_ERROR)
								&& (actionID.equals(INTERACTIVE_COMPILATION))) {
							new InteractiveCompilation(compilingMethods, tree, sourceFile, graphValue, compilationUtils);
							compilationUtils.refreshContent();
						}
						
						// If there is no problem in the graph transformation, for the general compilation
						else if ((graphValue.getReturnCode() == ExitStatus.NO_ERROR)
								&& (actionID.equals(GENERAL_COMPILATION))) {
							new GeneralCompilation(compilingMethods, tree, sourceFile, graphValue, compilationUtils);
							compilationUtils.refreshContent();
						}
						else if (graphValue.getReturnCode() != ExitStatus.NO_ERROR) {
							PkPlugin.getTraceabilityAPI().printErrorsOnDiagram(tree, false);
							PolychronyConsole
									.printError(("Error detected : " + CompilationUtils.ERROR_GRAPH + ExitStatus.get(
										graphValue.getReturnCode()).getLiteral()));
							compilationUtils.refreshErrors(sourceFile);
						}
					}
					else {
						PolychronyConsole.printError("Error detected : " + CompilationUtils.ERROR_TREE);
					}
				}
			}
		}
		else {
			MessageDialog.openError(new Shell(), CompilationUtils.ERROR_CODE_GENERATION,
				CompilationUtils.ERROR_NO_FILE_SELECTED);
		}
	}
	
	/**
	 * Method used to start the execution of a scenario
	 * 
	 * @param scenario
	 *        the scenario to execute
	 */
	public void executeScenario(Scenario scenario, CompilationUtils compilationUtils) {
		ExecuteScenarioJob execute = new ExecuteScenarioJob(sourceFile, scenario, compilationUtils);
		execute.setUser(true);
		execute.schedule();
	}
	
	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		if (selection instanceof StructuredSelection) {
			/*
			 * We know here that there is one and only one source file because it is specified in the plugin.xml file,
			 * at the following line : enablesFor="1"
			 */
			sourceFile = (IFile) ((StructuredSelection) selection).getFirstElement();
		}
	}
}
