/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.commands;

import org.eclipse.gef.commands.Command;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.CartesianProduct;
import fr.irisa.espresso.sme.ClockRelationOperator;
import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.Iterate;
import fr.irisa.espresso.sme.MultipleInputsOperator;
import fr.irisa.espresso.sme.OutputPort;
import fr.irisa.espresso.sme.SignalOperator;
import fr.irisa.espresso.sme.SmeFactory;

/**
 * 
 * Creation : 08 june 2006
 */
public class AddNewPortCommand extends Command {
	private GraphNode	signalOperatorNode;
	
	private GraphNode	inputPortNode;
	
	private GraphNode	outputPortNode;
	
	private Modeler		modeler;
	
	/**
	 * Constructor
	 * 
	 * @param element
	 *        the GraphNode that will contain the newly created Port
	 */
	public AddNewPortCommand(GraphNode element, Modeler editor) {
		super("Add new port");
		this.signalOperatorNode = element;
		this.modeler = editor;
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	public boolean canExecute() {
		return signalOperatorNode != null;
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		redo();
	}
	
	/**
	 * Add a new InputPort object to the MultipleInputsOperator model object and a new OutputPort if the model object is
	 * a CartesianProduct
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		ICreationUtils factory = modeler.getActiveConfiguration().getCreationUtils();
		SignalOperator sigop = (SignalOperator) Utils.getElement(signalOperatorNode);
		
		if (sigop instanceof CartesianProduct) {
			// create the model object
			OutputPort outputPort = SmeFactory.eINSTANCE.createOutputPort();
			((CartesianProduct) sigop).getOuts().add(outputPort);
			outputPort.setOperatorParent(sigop);
			
			// create the graphNode and its position property
			outputPortNode = (GraphNode) factory.createGraphElement(outputPort);
			signalOperatorNode.getContained().add(outputPortNode);
		}
		if (sigop instanceof MultipleInputsOperator) {
			// create the model object
			InputPort inputPort = SmeFactory.eINSTANCE.createInputPort();
			((MultipleInputsOperator) sigop).getIns().add(inputPort);
			inputPort.setOperatorParent(sigop);
			
			// create the graphNode and its position property
			inputPortNode = (GraphNode) factory.createGraphElement(inputPort);
			signalOperatorNode.getContained().add(inputPortNode);
		}
		else if (sigop instanceof Iterate) {
			// create the model object
			InputPort inputPort = SmeFactory.eINSTANCE.createInputPort();
			((Iterate) sigop).getIns().add(inputPort);
			inputPort.setOperatorParent(sigop);
			
			// create the graphNode and its position property
			inputPortNode = (GraphNode) factory.createGraphElement(inputPort);
			signalOperatorNode.getContained().add(inputPortNode);
		}
		else if (sigop instanceof ClockRelationOperator) {
			// create the model object
			InputPort inputPort = SmeFactory.eINSTANCE.createInputPort();
			((ClockRelationOperator) sigop).getIns().add(inputPort);
			inputPort.setOperatorParent(sigop);
			
			// create the graphNode and its position property
			inputPortNode = (GraphNode) factory.createGraphElement(inputPort);
			signalOperatorNode.getContained().add(inputPortNode);
		}
	}
	
	/**
	 * Remove the last added InputPort (and OutputPort)
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		SignalOperator sigop = (SignalOperator) Utils.getElement(signalOperatorNode);
		if (sigop instanceof CartesianProduct && outputPortNode != null) {
			((CartesianProduct) sigop).getOuts().remove(Utils.getElement(outputPortNode));
			signalOperatorNode.getContained().remove(outputPortNode);
		}
		if (inputPortNode != null) {
			if (sigop instanceof MultipleInputsOperator) {
				((MultipleInputsOperator) sigop).getIns().remove(Utils.getElement(inputPortNode));
				signalOperatorNode.getContained().remove(inputPortNode);
			}
			else if (sigop instanceof Iterate) {
				((Iterate) sigop).getIns().remove(Utils.getElement(inputPortNode));
				signalOperatorNode.getContained().remove(inputPortNode);
			}
			else if (sigop instanceof ClockRelationOperator) {
				((ClockRelationOperator) sigop).getIns().remove(Utils.getElement(inputPortNode));
				signalOperatorNode.getContained().remove(inputPortNode);
			}
		}
	}
}
