/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.commands;

import java.util.Collection;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.CompoundCommand;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.InputInstance;
import fr.irisa.espresso.sme.ModelInstance;
import fr.irisa.espresso.sme.OutputInstance;
import fr.irisa.espresso.sme.ParameterInstance;
import fr.irisa.espresso.sme.SignalElement;

/**
 * This command is used to create the ports of a ModelInstance
 * 
 * Creation : 26 july 2006
 */
public class CreatePortInstanceCommand extends CompoundCommand {
	/** The GraphNode that should be updated */
	private GraphNode		container;
	
	/** The EditDomain associated */
	private EditDomain		editDomain;
	
	private ICreationUtils	creationUtils;
	
	/** Lists containing all new EObjects to add to the container graphNode */
	private EList<EObject>	newObjects	= new BasicEList<EObject>();
	
	/**
	 * Constructor
	 * 
	 * @param domain
	 *        the EditDomain
	 * @param container
	 *        the graphNode in which the new InputInstance will be added
	 * @param creationUtils
	 *        the creationUtils factory
	 * @param in
	 *        the InputInstance to add
	 */
	public CreatePortInstanceCommand(EditDomain domain, GraphNode container, ICreationUtils creationUtils,
			InputInstance in) {
		super("Add an InputInstance Command");
		
		this.editDomain = domain;
		this.container = container;
		this.creationUtils = creationUtils;
		
		if (in != null)
			newObjects.add(in);
		
		initialize();
	}
	
	/**
	 * Constructor
	 * 
	 * @param domain
	 *        the EditDomain
	 * @param container
	 *        the graphNode in which the new OutputInstance will be added
	 * @param creationUtils
	 *        the creationUtils factory
	 * @param in
	 *        the OutputInstance to add
	 */
	public CreatePortInstanceCommand(EditDomain domain, GraphNode container, ICreationUtils creationUtils,
			OutputInstance out) {
		super("Add an OutputInstance Command");
		
		this.editDomain = domain;
		this.container = container;
		this.creationUtils = creationUtils;
		
		if (out != null)
			newObjects.add(out);
		
		initialize();
	}
	
	/**
	 * Constructor
	 * 
	 * @param domain
	 *        the EditDomain
	 * @param container
	 *        the graphNode in which the new ParameterInstance will be added
	 * @param creationUtils
	 *        the creationUtils factory
	 * @param in
	 *        the ParameterInstance to add
	 */
	public CreatePortInstanceCommand(EditDomain domain, GraphNode container, ICreationUtils creationUtils,
			ParameterInstance par) {
		super("Add a ParameterInstance Command");
		
		this.editDomain = domain;
		this.container = container;
		this.creationUtils = creationUtils;
		
		if (par != null)
			newObjects.add(par);
		
		initialize();
	}
	
	/**
	 * Constructor
	 * 
	 * @param domain
	 *        the EditDomain
	 * @param container
	 *        the graphNode in which the new InputInstance will be added
	 * @param creationUtils
	 *        the creationUtils factory
	 * @param list
	 *        the list of Port Instances (ParameterInstance, InputInstance, or OutputInstance) to add
	 */
	public CreatePortInstanceCommand(EditDomain domain, GraphNode container, ICreationUtils creationUtils,
			Collection<? extends SignalElement> list) {
		super("Add a list of Port Instance Command");
		
		this.editDomain = domain;
		this.container = container;
		this.creationUtils = creationUtils;
		
		if (list != null)
			newObjects.addAll(list);
		
		initialize();
	}
	
	private void initialize() {
		EObject eObject = Utils.getElement(container);
		if (eObject instanceof ModelInstance) {
			for (EObject port : newObjects) {
				// check if the element is already present
				GraphNode node = getGraphicalRepresentation(port);
				
				if (node == null) {
					// create the new graphNode and add it to the parentNode
					node = (GraphNode) creationUtils.createGraphElement(port);
					
					if (port instanceof OutputInstance)
						add(new CreateGraphNodeCommand(editDomain, node, container, null, node.getSize(),
							PositionConstants.RIGHT));
					else if (port instanceof InputInstance || port instanceof ParameterInstance)
						add(new CreateGraphNodeCommand(editDomain, node, container, null, node.getSize(),
							PositionConstants.LEFT));
				}
			}
		}
	}
	
	/**
	 * Get the graphNode associated with the model object in the current selected editPart
	 * 
	 * @param modelObject
	 *        the model object to display
	 * @return the GraphNode or null if it does not exist
	 */
	private GraphNode getGraphicalRepresentation(EObject modelObject) {
		for (DiagramElement elt : container.getContained()) {
			if (elt instanceof GraphNode) {
				if (Utils.getElement((GraphNode) elt) == modelObject)
					return (GraphNode) elt;
			}
		}
		return null;
	}
	
}
