/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.commands;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Input;
import fr.irisa.espresso.sme.InputInstance;
import fr.irisa.espresso.sme.InterfaceDefinition;
import fr.irisa.espresso.sme.ModelInstance;
import fr.irisa.espresso.sme.Output;
import fr.irisa.espresso.sme.OutputInstance;
import fr.irisa.espresso.sme.Parameter;
import fr.irisa.espresso.sme.ParameterInstance;

/**
 * 
 * Creation : 08 june 2006
 */
public class PositionInstanceCommand extends Command {
	private GraphNode	container;
	
	private boolean		withParameters;
	
	/**
	 * Constructor
	 * 
	 * @param element
	 *        the GraphNode that represent the ModelInstance on which we will operate
	 */
	public PositionInstanceCommand(GraphNode element, boolean withParameters) {
		super("Position the port instance in a model instance");
		this.container = element;
		this.withParameters = withParameters;
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	public boolean canExecute() {
		return container != null && (Utils.getElement(container) instanceof ModelInstance);
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		redo();
	}
	
	/**
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		ModelInstance mi = (ModelInstance) Utils.getElement(container);
		InterfaceDefinition id = mi.getReferredInterface();
		if (id == null)
			return;
		
		EList<Parameter> params = id.getParameters();
		EList<Input> ins = id.getInputs();
		EList<Output> outs = id.getOutputs();
		
		int nbParams = params.size(), nbIns = ins.size(), nbOuts = outs.size();
		if (withParameters)
			nbIns += nbParams;
		
		int height;
		if (container.getSize() == null)
			height = container.getSize().height;
		else
			height = Math.max(70, 20 * Math.max(nbIns, nbOuts));
		
		if (nbIns > 0 || nbOuts > 0) {
			// place the ParameterInstance, InputInstance, and OutputInstance
			int space_in = height / (nbIns > 0 ? nbIns : 1), space_out = height / (nbOuts > 0 ? nbOuts : 1);
			int init_in = (space_in - 10) / 2, init_out = (space_out - 10) / 2;
			
			for (DiagramElement elt : container.getContained()) {
				if (elt instanceof GraphElement) {
					EObject obj = Utils.getElement((GraphElement) elt);
					int index;
					if (obj instanceof ParameterInstance) {
						// Search index of the parameter in the InterfaceDefinition parameter list
						index = params.indexOf(((ParameterInstance) obj).getParameter());
						((GraphElement) elt).setPosition(new Point(0, init_in + (index * space_in)));
					}
					else if (obj instanceof InputInstance) {
						// Search index of the parameter in the InterfaceDefinition parameter list
						index = ins.indexOf(((InputInstance) obj).getInput());
						if (withParameters && index != -1)
							index += nbParams; // Inputs are positionned after Parameters
						((GraphElement) elt).setPosition(new Point(0, init_in + (index * space_in)));
					}
					else if (obj instanceof OutputInstance) {
						// Search index of the parameter in the InterfaceDefinition parameter list
						index = outs.indexOf(((OutputInstance) obj).getOutput());
						((GraphElement) elt).setPosition(new Point(0, init_out + (index * space_out)));
					}
				}
			}
		}
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {

	}
}
