/***Polychrony License******************************************
 *                                                             *
 * Copyright: IRISA/INRIA Rennes - ESPRESSO team.              *
 * This software is under the terms of the CeCILL-B License.   *
 * http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html *
 *                                                             *
 ***************************************************************/
#ifndef __PROCESSDESCRIPTOR_H__
#define __PROCESSDESCRIPTOR_H__

#include "TYPES.h"


                              /*============================*/
                              /* Author: A. GAMATIE         */
                              /* Last modified 03/13/2003   */
                              /*============================*/

/**************************************************************************************/
/**************************  class PROCESS DESCRIPTOR (headers) ***********************/
/**************************************************************************************/
/* The following class defines process descriptors used by the scheduler. This class  */
/* is still under implementation. So no guarantee is given that it works correctly.   */
/* However, it should not be very far from what you expect. :-) Please, send any      */
/* remark at the following address agamatie@irisa.fr                                  */
/**************************************************************************************/


/**************************************************************************************/
/*******************************   PROCESS DESCRIPTOR : BEGIN *************************/
/**************************************************************************************/

typedef class ProcessDescriptor *ptProcessDescriptor;
class ProcessDescriptor
{
  private:
   ProcessStatus_type ProcessStatus;                                /* Process status */  
   logical CreationIndicator;                                      /* Creation indicator */ 
   logical WaitIndicator;                               /* Waiting on resource indicator */
   Comm_ComponentID_type ResourceUsed[MAX_RESOURCES];   
                                           /* Resources currently used by the process */
   SystemTime_type WaitingTime;                         /* current time counter value */ 
   Message_type Message;                                         /* retrieved message */
   logical SuspensionIndicator;   /* the current process has been suspended by another by
                                                 another process which is not himself */



  public:
  ProcessDescriptor(ProcessName_type P_name,
                    SystemAddress_type P_entrypoint, 
                    StackSize_type P_stacksize, 
                    Priority_type P_basepriority, 
                    SystemTime_type P_period, 
                    SystemTime_type P_timecapacity, 
                    Deadline_type P_deadline);
  /* Constructor. */

    
  ProcessDescriptor();
  /* Constructor. */


  ProcessDescriptor(ProcessDescriptor& desc);
  /* Constructor. */


  ~ProcessDescriptor();
  /* Destructor. */


  void setProcessName(ProcessName_type P_name);
  /* Fills the name in the descriptor. */
  void getProcessName(ProcessName_type& P_name);
  /* "P_name" denotes the content of the field Name in the descriptor. */


  void setEntryPoint(SystemAddress_type P_entryPt);
  /* Fills the entry point in the descriptor. */
  SystemAddress_type getEntryPoint();
  /* Returns the entry point value associated with the descriptor. */


  void setStackSize(StackSize_type P_stacksize);
  /* Fills the stack size in the descriptor. */
  StackSize_type getStackSize();
  /* Returns the stack size associated with the descriptor. */

  
  void setBasePriority(Priority_type P_basepriority);
  /* Fills the base priority in the descriptor. */
  Priority_type getBasePriority();
  /* Returns the base priority level associated with the descriptor. */


  void setPeriod(SystemTime_type P_period);
  /* Fills the period in the descriptor. */
  SystemTime_type getPeriod();
  /* Returns the period value associated with the descriptor. */
 

  void setTimeCapacity(SystemTime_type P_timecap);
  /* Fills the time capacity in the descriptor. */
  SystemTime_type getTimeCapacity();
  /* Returns the time capacity associated with the descriptor. */

     
  void setDeadline(Deadline_type P_deadline);
  /* Fills the deadline type (HARD/SOFT) in the descriptor. */
   Deadline_type getDeadline();
  /* Returns the deadline type associated with the descriptor. */


  void setCurrentPriority(Priority_type P_currentpriority);
  /* Fills the current priority value in the descriptor. */
  Priority_type getCurrentPriority();
  /* Returns the current priority value associated with the descriptor. */

 
  void setDeadlineTime(SystemTime_type P_dltime);
  /* Fills the deadline time in the descriptor. */
  SystemTime_type getDeadlineTime();
  /* Returns the deadline time associated with the descriptor. */


  void setProcessState(ProcessState_type P_state);
  /* Fills the state in the descriptor. */
  ProcessState_type getProcessState();
  /* Returns the state associated with the descriptor. */


  void setWaitIndicator(logical P_waitind);
  /* Fills the wait indicator in the descriptor. */
  logical getWaitIndicator();
  /* Returns the wait indicator value associated with the descriptor. */


  void setProcessAttributes(ProcessAttributes_type Attributes);
  /* Fills the attributes in the descriptor. */
  ProcessAttributes_type getProcessAttributes();
  /* Returns the attributes associated with the descriptor. */


  void setProcessStatus(ProcessStatus_type vStatus);
  /* Fills the status in the descriptor. */
  ProcessStatus_type getProcessStatus();
  /* Returns the status associated with the descriptor. */


  void setCreationIndicator(logical created);
  /* Fills the creation indicator in the descriptor. */
  logical getCreationIndicator();
  /* Returns the creation indicator value associated with the descriptor. */


  void setWaitingTime(SystemTime_type delay);
  /* Fills the waiting time in the descriptor. */
  SystemTime_type getWaitingTime();
  /* Returns the waiting time associated with the descriptor. */


  void incrementWaitingTime();
  void incrementWaitingTimeProfiling(double val);
  /* Increments the waiting time respectively by one and val. */


  void decrementWaitingTime();
  void decrementWaitingTimeProfiling(double val);
  /* Decrements the waiting time respectively by one and val. */


  logical nullWaitingTime();
  /* The waiting time is not strictly positive. */


  void insertResource(Comm_ComponentID_type component);
  /* Inserts a resource identifier in the list of resources associated
     with the descriptor. */
  void removeResource(Comm_ComponentID_type component);
  /* Removes a resource identifier from the list of resources associated
     with the descriptor. */
  void removAllResource();
  /* Removes all resources from the list of resources associated with 
     the descriptor. */


  void setMessage(Message_type mess);
  /* Fills the message in the descriptor. */
  Message_type getMessage();
  /* Returns the message associated with the descriptor. */


  void setSuspensionIndicator(logical suspension);
  /* Fills the suspension indicator in the descriptor. */
  logical getSuspensionIndicator();
  /* Returns the suspension indicator value associated with the descriptor. */

};

/********************************************************************************************/
/*********************************   PROCESS DESCRIPTOR : END *******************************/
/********************************************************************************************/


#endif // __PROCESSDESCRIPTOR_H__
