/***Polychrony License******************************************
 *                                                             *
 * Copyright: IRISA/INRIA Rennes - ESPRESSO team.              *
 * This software is under the terms of the CeCILL-B License.   *
 * http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html *
 *                                                             *
 ***************************************************************/

#define __END_STAMP__   -1
#define __SYNC_STAMP__  -2
#define __ERROR_STAMP__ -3

class Task ; // used forward

/*******************************************/
/* declaration of the TaskFifo class      */

class TaskFifo
{
public:
  // 
  TaskFifo ( int task_number, Task ** task_list ) ;
  ~TaskFifo ( ) ;
  int GetTasksNumber ( ) ;  // return the number of tasks
  Task * GetTask ( int ) ;  // return a task
  void ReInit ( ) ;  // initializes the fifo
  void Push ( Task * t ) ;  // pushes a task into the fifo
  Task * Pop ( ) ;  // pops a task from the fifo (NULL when empty)
  int IsEmpty ( ) ;  // return 0 when the fifo is empty, 1 otherwise
private:
  Task ** _TaskList ;  // table of tasks
  Task ** _TaskFifo ;  // fifo of tasks
  int _TasksNumber ;  // number of tasks
  int _FillIndex ;  // index where tasks are to be put into the fifo
  int _ExtractIndex ;  // index where tasks are to be retreived from the fifo
  int _InsideNumber ;  // number of tasks in the fifo
};

typedef int (*GetNewInputFunc) ( int *, char ** ) ;
typedef void (*UpdateInputFunc) ( int, char * ) ;

typedef void (*SendMessFunc) ( int, int ) ;
typedef void (*BroadCastMessFunc) ( int ) ;

/*******************************************/
/* declaration of the Scheduler classes    */

class Scheduler : public TaskFifo
{
public:
  // 
  Scheduler(int task_number, 
	    Task ** task_list, 
	    GetNewInputFunc GetNewInput, 
	    UpdateInputFunc UpdateFunc,
	    SendMessFunc SendMess,
	    BroadCastMessFunc BroadCastMess,
	    char * Name);
  virtual int Iterate ( ) = 0 ;  // executes one logical instant
  void Initialize ( ) ;  // initializes all the tasks
  // debugging functions
  void SetVerbose ( ) ;  // enables debugging functions
  int IsVerbose ( ) ;  // testing the debugging mode
  void Trace ( char * string ) ;  // debugging functions
  void TraceInt ( char * string, int n ) ;
  void TraceStr ( char * string, char * s ) ;
  void TraceStrInt ( char * string, char * s, int n ) ;
protected:
  GetNewInputFunc _GetNewInput ;  // Function called to read new inputs
  UpdateInputFunc _UpdateInput ;  // Function called to modify memory with inputs
  SendMessFunc _SendMess;  // Function called to send a message to a processor
  BroadCastMessFunc _BroadCastMess;  // Function called to send a message to all the processors
  int _Weight ;  // number of tasks to be executed
  void NewInstant ( ) ;  // Trace a new instant
  enum SchedulerState { __NORMAL__, __ERROR__, __END__ } ;// State of the scheduler while iterating
private:
  int _Verbose ;  // Debugging flag
  FILE * _TraceFile ;  // Tracing file
  char * _Name ;  // string used for tracing
  int _InstantNumber ;// number of the current instant
};

class MasterScheduler : public Scheduler
{
public:
  // 
  MasterScheduler(int task_number, Task ** task_list, 
		  GetNewInputFunc GetNewInput, 
		  UpdateInputFunc UpdateFunc,
		  SendMessFunc SendMess,
		  BroadCastMessFunc BroadCastMess,
		  int slaves_number,
		  char * Name);
  virtual int Iterate ( ) ;  // executes one logical instant
private:
  void ReInit ( ) ;  // reinitializations
  int Scan ( ) ;  // scans a new input
  void ScanEnd ( ) ;  // scans an __END__ message
  void TraceAndStop ( ) ;  // called on timeouts
  int _SlavesNumber ;  // number of slaves to be synchronized at each instant
  int _SyncWeight ;  // number of slaves still to be synchronized
};

class SlaveScheduler : public Scheduler
{
public:
  // 
  SlaveScheduler(int task_number, 
		 Task ** task_list, 
		 GetNewInputFunc GetNewInput, 
		 UpdateInputFunc UpdateFunc,
		 SendMessFunc SendMess,
		 BroadCastMessFunc BroadCastMess,
		 int master,
		 char * Name);
  virtual int Iterate ( ) ;  // executes one logical instant

private:
  void ReInit ( ) ;  // reinitializations
  int Scan ( ) ;  // scans a new input
  void TraceAndStop ( ) ;  // called on timeouts
  int _Master ;  // master processor
};

//typedef int (*IterFunc) ( ) ;
typedef void (*IterFunc) ( ) ;

// typedef int (*InitFunc) ( ) ;

/*******************************************/
/* declaration of the Task class           */

class Task 
{
public:
  Task ( 
	Scheduler * Sched,
	IterFunc Iter, 
	//InitFunc Init, 
	int PredNumber, 
	int SuccNumber, 
	Task * Succ[],
	char * Name) ;
  Task ( 
	Scheduler * Sched,
	IterFunc Iter, 
	//	InitFunc Init, 
	int PredNumber,
	char * Name ) ;
  void Initialize ( ) ;
  void ReInit ( ) ;
  void Iterate ( ) ;
  void Start ( ) ;
  void SetReady ( ) ;
  void Select ( ) ;
  void Terminate ( ) ;
  void DecCounter ( );
  void Trace ( char * string ) ;
  void TraceBool ( char * string, int b ) ;
  int IsNotBegan ( ) ;
  int GetCounter ( ) ;
  char * GetName ( ) ;
  int GetSuccNumber ( ) ;
  Task * GetSucc ( int n ) ;
private:
  enum TaskState { _not_began, _ready, _running, _terminated } ;
  Scheduler * _Sched;
  Task ** _Succ ;
  int _SuccNumber ;
  int _PredNumber ;
  int _Counter ;
  TaskState _State ;
  IterFunc _Iter ;
  //  InitFunc _Init ;
  char * _Name ;
};

