/**
   Authors: Loïc Besnard, François Fabre, Thierry Gautier.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - EPL License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony SSME Platform (website: http://www.irisa.fr/espresso/polychrony).

   Polychrony SSME Platform is free software; you can redistribute it and/or modify it under
   the terms of the EPL as published by the Eclipse Foundation.

   Polychrony SSME Platform are distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the Eclipse Public License - v 1.0 for more details.

   You should have received a copy of the Eclipse Public License - v 1.0
   along with Polychrony SSME Platform; see the file COPYING. If not, consult the url
   http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse Foundation, Inc.
   102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
*/

package fr.irisa.espresso.sme.polychrony;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;

import fr.irisa.espresso.sme.polychrony.preferences.SmePreferenceConstants;
import fr.irisa.espresso.sme.polychrony.traceability.TreesInterface;

/**
 * This file contains all SIGNAL services functions concerning the transformations of a SIGNAL graph.
 */
public class PKServices {
	private final static String		NATIVE_LIB_NOT_FOUND					= "The native library for Polychrony was not found.";
	private final static String		NO_NATIVE_RESOURCES						= "Cannot find native resources to initialize the compiler";
	private final static String		TYPE_TABLE_MAX_SIZE_MUST_BE_POSITIVE	= "The Type Table maximum size must be strictly positive. See Preferences->Polychrony page.";
	private final static String		TREE_STRUCTURE_SIZE_MUST_BE_POSITIVE	= "The Memory size for the tree structure must be strictly positive. See Preferences->Polychrony page.";
	private final static String		mess4									= "The Memory size for the native string structure must be strictly positive. See Preferences->Polychrony page.";
	
	/*
	 * Loads the Signal Java-C interface library to access to compilation and verification services.
	 */
	static {
		try {
			System.loadLibrary("SignalJavaC_API");
		}
		catch (UnsatisfiedLinkError ule) {
			PkPlugin.getDefault().getLog()
					.log(new Status(IStatus.ERROR, PkPlugin.PLUGIN_ID, NATIVE_LIB_NOT_FOUND, ule));
		}
	}
	
	public final static PKServices	INSTANCE								= new PKServices();
	private SignalParser			parser;
	private SignalTreePrinter		printer;
	private SignalTreeBuilting		builder;
	private PKTrees					treeAPI;
	private SignalTreeComment		comment;
	private TreesInterface			treesInterface;
	private SignalTreeExploring		explorer;
	
	/**
	 * Constructor
	 */
	private PKServices() {
		String fixedPath = null;
		try {
			// Search one of the resource needed by the native part
			URL entry = PkPlugin.getDefault().getBundle().getResource("messignal.0");
			if (entry != null) {
				// Localize the file on the file system
				String path = FileLocator.toFileURL(entry).getPath();
				// Obtain the path to its parent directory
				fixedPath = new File(path).getParentFile().getPath();
			}
			else
				PkPlugin.getDefault().getLog().log(new Status(IStatus.ERROR, PkPlugin.PLUGIN_ID, NO_NATIVE_RESOURCES));
		}
		catch (IOException ioe) {
			PkPlugin.getDefault().getLog().log(
				new Status(IStatus.ERROR, PkPlugin.PLUGIN_ID, 0, NO_NATIVE_RESOURCES, ioe));
		}
		if (fixedPath != null) {
			try {
				newMachine();
				IPreferenceStore pref = PkPlugin.getDefault().getPreferenceStore();
				// Initialization of the necessary Signal environment variables
				setEnv(EnvironmentConstants.VERSION, EnvironmentConstants.VERSION_VALUE);
				setEnv(EnvironmentConstants.BIN_PATH, fixedPath);
				String value = pref.getString(SmePreferenceConstants.LIBRARY_PATH);
				if (!("".equals(value)))
					value += File.pathSeparator;
				value += fixedPath + File.separator + "lib" + File.separator + "Std";
				setEnv(EnvironmentConstants.LIBRARY_PATH, value);
				
				int val = pref.getInt(SmePreferenceConstants.TYPE_TABLE_SIZE);
				if (val <= 0) {
					PkPlugin.getDefault().getLog().log(
						new Status(IStatus.WARNING, PkPlugin.PLUGIN_ID, TYPE_TABLE_MAX_SIZE_MUST_BE_POSITIVE));
					val = EnvironmentConstants.TYPE_TABLE_SIZE_DEFAULT_VALUE;
				}
				setEnv(EnvironmentConstants.TYPE_TABLE_SIZE, "" + val);
				
				val = pref.getInt(SmePreferenceConstants.T_MEM_ARB);
				if (val <= 0) {
					PkPlugin.getDefault().getLog().log(
						new Status(IStatus.WARNING, PkPlugin.PLUGIN_ID, TREE_STRUCTURE_SIZE_MUST_BE_POSITIVE));
					val = EnvironmentConstants.TREE_MEMORY_SIZE_DEFAULT_VALUE;
				}
				setEnv(EnvironmentConstants.TREE_MEMORY_SIZE, "" + val);
				
				val = pref.getInt(SmePreferenceConstants.CHAIN_MEM_ARB);
				if (val <= 0) {
					PkPlugin.getDefault().getLog().log(new Status(IStatus.WARNING, PkPlugin.PLUGIN_ID, mess4));
					val = EnvironmentConstants.STRING_MEMORY_SIZE_DEFAULT_VALUE;
				}
				setEnv(EnvironmentConstants.STRING_MEMORY_SIZE, "" + val);
				
				initMachine();
			}
			catch (Exception e) {
				e.printStackTrace();
			}
		}
		parser = new SignalParser();
		printer = new SignalTreePrinter();
		builder = new SignalTreeBuilting();
		treeAPI = new PKTrees();
		comment = new SignalTreeComment();
		treesInterface = new TreesInterface();
		explorer = new SignalTreeExploring();
	}
	
	/**
	 * @return
	 */
	public SignalParser getParser() {
		return parser;
	}
	
	/**
	 * @return the printer
	 */
	public SignalTreePrinter getPrinter() {
		return printer;
	}
	
	/**
	 * @return the tree builder
	 */
	public SignalTreeBuilting getTreeBuilder() {
		return builder;
	}
	/**
	 * @return the tree builder
	 */
	public SignalTreeExploring getTreeExplorer() {
		return explorer;
	}
	/**
	 * @return the treeApi
	 */
	public PKTrees getTreeAPI() {
		return treeAPI;
	}
	
	/**
	 * @return the tree comment
	 */
	public SignalTreeComment getCommentAPI() {
		return comment;
	}
	
	public TreesInterface getTraceabilityAPI() {
		return treesInterface;
	}
	
	/*******************************************************************************************************************
	 * How to manage an interface
	 * 
	 * C function file : pKServicesAPI_g.h
	 ******************************************************************************************************************/
	
	/**
	 * Instantiate a new PKService Machine This method must be called before calling initMachine()
	 */
	private synchronized native void newMachine();
	
	/**
	 * Initialize the use of the API.
	 * 
	 * @return the pkService API instance
	 */
	private synchronized native void initMachine();
	
	public synchronized native void resetMachine();
	
	/**
	 * This method enables/disables the warning flag. This method has to be called before any tree or graph
	 * transformation to be used.
	 * 
	 * @param enable
	 *        true to display the warnings, false otherwise.
	 */
	public synchronized native void displayWarnings(boolean enable);
	
	/**
	 * Finalyzing the use of the PKServices
	 */
	public synchronized native void finalize();
	
	/*******************************************************************************************************************
	 * Applying transformations on the tree
	 * 
	 * C function file : pKServicesAPImkGraph_g.h
	 ******************************************************************************************************************/
	
	/**
	 * Abstract tree representation to graph translation (entry point for the GUI integrated compiler). The following
	 * steps are applied :
	 * <ul>
	 * <li>translating to kernel-tree representation.</li>
	 * <li>production the graph.</li>
	 * 
	 * @param tree
	 *        tree node built by the getDeclarationAST of a Model Declaration or a Module.
	 * @param retTreeError
	 *        true to return error-code for the tree.
	 * @param paramTree
	 *        abstract tree of the parameters file.
	 * @param retParamTreeError
	 *        true to return error-code of the parser for paramTree.
	 * @param dirName
	 *        the complete name of the directory where to generate the compiling result.
	 * @param generateLISFile
	 *        true to produce the annotated original Signal program (X_LIS.SIG file) when errors or not.
	 * @param vgraph
	 *        (out) : the graph of an instance of the model defined in srcfile.
	 * @return a non null value when errors are detected, 0 otherwise.
	 */
	public synchronized native GraphValue tree2DCG(long tree, boolean retTreeError, long paramTree,
			boolean retParamTreeError, String outputDirectory, boolean generateLISFile);
	
	public synchronized native boolean printSignalLIS(long tree, long params);
	
	/**
	 * This method translates an abstract syntax tree corresponding to a Signal program into its SME representation,
	 * according to the SME meta-model.
	 * 
	 * @param tree
	 *        the abstract syntax tree to translate.
	 * @param outputDirectory
	 *        the directory in which the SME file is generated. the name of the file is given by the name of the Signal
	 *        model, suffixed by ".sme".
	 * @return the value 0 if the operation succeeds, greater than 0 otherwise.
	 * 
	 */
	public synchronized native int signal2SME(long tree, String outputDirectory);
	
	/**
	 * This method translates the representation of a textual Signal program into its SME representation, according to
	 * the SME meta-model.
	 * 
	 * @param srcFile
	 *        the file that contains the Signal program to translate.
	 * @param outputDirectory
	 *        the directory in which the SME file is generated. the name of the file is given by the name of the Signal
	 *        model, suffixed by ".sme".
	 * @return the value 0 if the operation succeeds, greater than 0 otherwise.
	 * 
	 */
	public synchronized native int signal2SME(String srcFile, String outputDirectory);
	
	/*******************************************************************************************************************
	 * Applying transformations on the graph
	 * 
	 * C function file : pKServicesAPIGraph_g.h
	 ******************************************************************************************************************/
	
	/**
	 * @param graphNode
	 * @param mainFile
	 * @param bodyFile
	 * @param ioFile
	 * @return
	 */
	public synchronized native boolean genCodeC(long graphNode, boolean mainFile, boolean bodyFile, boolean ioFile);
	
	/**
	 * @param graphNode
	 * @param mainFile
	 * @param bodyFile
	 * @param ioFile
	 * @return
	 */
	public synchronized native boolean genCodeCPP(long graphNode, boolean mainFile, boolean bodyFile, boolean ioFile);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean genCodeCPPSeparate(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean genCodeCSeparate(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean DotUnparse(long graphNode);
	
	/**
	 * @param graphNode
	 * @param mainFile
	 * @param bodyFile
	 * @param ioFile
	 * @return
	 */
	public synchronized native boolean genCodeJava(long graphNode, boolean mainFile, boolean bodyFile, boolean ioFile);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean genCodeJavaSeparate(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean expressionExpand(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean unparse(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native long produce(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native long produceFlat(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean SynDExUnparse(long graphNode);
	
	/**
	 * @param graphNode
	 * @param mainFile
	 * @param bodyFile
	 * @param ioFile
	 * @return
	 */
	public synchronized native boolean genCodeJavaThreads(long graphNode, boolean mainFile, boolean bodyFile,
			boolean ioFile);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean IGraph2Vhdl(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean SigaliPrintExtraction(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native long produceInterfaceAbstraction(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native long buildAbstraction(long graphNode);
	
	/**
	 * @param graphNode
	 * @param delayMultiplexing
	 */
	public synchronized native void memoryOptimizations(long graphNode, boolean delayMultiplexing);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native GraphValue DCGPoly2DCGBool(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean transformBooleanClocks2Events(long graphNode);
	
	/**
	 * @param graphNode
	 */
	public synchronized native void separateBooleansOthers(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native GraphValue DCGBasic2DCGPoly(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean setIntrinsicClocks(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean dependenceReinforcing(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean addExceptionsForConstraints(long graphNode);
	
	/**
	 * @param graphNode
	 * @param debug
	 * @return
	 */
	public synchronized native boolean hasStaticCycle(long graphNode, boolean debug);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean splitDefaultDefinition(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean setDelayMemoryOverloading(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean moveIn2OutDelays(long graphNode);
	
	/**
	 * @param graphNode
	 * @param vlevel
	 */
	native void computeClockExclusions(long graphNode, int vlevel);
	
	/**
	 * @param graphNode
	 * @param level
	 */
	public synchronized void computeClockExclusions(long graphNode, ExclusiveClockLevelKind level) {
		computeClockExclusions(graphNode, level.getValue());
	}
	
	/**
	 * @param graphNode
	 * @param asSubGraph
	 * @return
	 */
	public synchronized native boolean expandModelReference(long graphNode, boolean asSubGraph);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean breakCPG(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean checkExplicitTyping(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean printCHStatistics(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean phylumClustering(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean buildClockProcessInterface(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean hasUnsolvedClockConstraints(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean delayTreeBuild(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean mergePartialDefinitions(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean applyTabularMorphism(long graphNode);
	
	/**
	 * @param graphNode
	 * @param clockOfDependences
	 * @return
	 */
	public synchronized native boolean getIOClockSubHierarchy(long graphNode, boolean clockOfDependences);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean nonProductiveSignalElimination(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean removeSignalEqualities(long graphNode);
	
	/**
	 * @param graphNode
	 * @param genCode
	 * @return
	 */
	public synchronized native boolean separatedCompilingPoly(long graphNode, boolean genCode);
	
	/**
	 * @param graphNode
	 * @param alwaysCode
	 * @param check
	 * @param delayMultiplexing
	 * @return
	 */
	public synchronized native boolean Serializing(long graphNode, boolean btotalOrder, boolean alwaysCode, boolean check,
			boolean delayMultiplexing);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean setDataDependences(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean unparseAbstraction(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean splitDataStructure(long graphNode);
	
	/**
	 * @param graphNode
	 */
	public synchronized native void separateStatesOthers(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean computeProductiveClockStateVar(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native GraphValue DCGPoly2DCGFlat(long graphNode);
	
	/**
	 * @param graphNode
	 * @param subGraph
	 * @return
	 */
	public synchronized native long produceSubgraph(long graphNode, int subGraph);
	
	/**
	 * @param graphNode
	 * @param maxRef
	 * @return
	 */
	public synchronized native boolean substituteExpr2Var(long graphNode, int maxRef);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean computeIOTransitiveClosure(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean removeSignalClones(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean computeProductiveClock(long graphNode);
	
	/**
	 * @param graphNode
	 * @return
	 */
	public synchronized native boolean LustreUnparse(long graphNode);
	
	/*******************************************************************************************************************
	 * Parse SIGNAL file
	 * 
	 ******************************************************************************************************************/
	
	/**
	 * This method parses a SIG source file and a parameter file and returns the corresponding graph. Take care, here,
	 * it is not an AST, but the tree2Graph method has already been called.
	 * 
	 * @param srcfile
	 *        the SIG file
	 * @param paramfile
	 *        the parameter file, or the empty string is none.
	 * @return the graph representing the files
	 */
	public synchronized native long parse(String srcFile, String paramFile);
	
	/*******************************************************************************************************************
	 * Set environment variables needed by the compiler THOSE NEEDED BY THE COMPILER HAS TO BE SET AFTER THE INIT
	 * MACHINE !!!
	 * 
	 ******************************************************************************************************************/
	
	/**
	 * @param key
	 *        a string to identify the environment variable
	 * @param value
	 *        the value associated to the key
	 */
	public synchronized native void setEnv(String key, String value);
	
	/**
	 * @param key
	 *        a string to identify the environment variable
	 * @return the value of the environment variable identified by <code>key</code> if it exists, null otherwise.
	 */
	public synchronized native String getEnv(String key);
	
}
