/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.polychrony.preferences;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.PathEditor;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import fr.irisa.espresso.sme.polychrony.EnvironmentConstants;
import fr.irisa.espresso.sme.polychrony.PkPlugin;

/**
 * A preference page to configure all preferences of the Sme editor.
 */
public class SmePreferencePage extends PreferencePage implements IWorkbenchPreferencePage {
	private IPreferenceStore	ps	= PkPlugin.getDefault().getPreferenceStore();
	
	/** Field to store paths to search Polychrony libraries */
	private PathEditor			libraryPath;
	
	/** Field to specify the maximum number of entry in the table of types */
	private IntegerFieldEditor	typeTableSize;
	
	/** Field to specify the memory size to store Polychrony tree internal structure information */
	private IntegerFieldEditor	treeMemorySize;
	
	/** Field to specify the memory size to store Polychrony string information */
	private IntegerFieldEditor	stringMemorySize;
	
	/**
	 * Creates this preference page contents.
	 * 
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		// Creates the container composite
		Composite container = new Composite(parent, SWT.NONE);
		GridLayout containerLayout = new GridLayout();
		containerLayout.marginWidth = 0;
		containerLayout.marginHeight = 0;
		container.setLayout(containerLayout);
		container.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		createHeader(container);
		
		createFields(container);
		
		initDefaultValues();
		loadPreferences();
		
		return container;
	}
	
	/**
	 * Set the default value for each field of the Polychrony Preference page
	 */
	private void initDefaultValues() {
		String fixedPath = null;
		try {
			// Searchs one of the resource needed by the native part
			URL entry = PkPlugin.getDefault().getBundle().getResource("messignal.0");
			if (entry != null) {
				// Localizes the file on the file system
				String path = FileLocator.toFileURL(entry).getPath();
				// Obtains the path to its parent directory
				fixedPath = new File(path).getParentFile().getPath();
			}
			else
				PkPlugin.getDefault().getLog().log(
					new Status(IStatus.ERROR, PkPlugin.PLUGIN_ID,
						"Cannot find native resources to initialize the Library Path"));
		}
		catch (IOException ioe) {
			PkPlugin.getDefault().getLog().log(
				new Status(IStatus.ERROR, PkPlugin.PLUGIN_ID, 0,
					"Cannot find native resources to initialize the Library Path", ioe));
		}
		
		if (fixedPath != null)
			ps.setDefault(SmePreferenceConstants.LIBRARY_PATH, fixedPath + File.separator + "lib" + File.separator
					+ "Std");
		ps.setDefault(SmePreferenceConstants.TYPE_TABLE_SIZE, EnvironmentConstants.TYPE_TABLE_SIZE_DEFAULT_VALUE);
		ps.setDefault(SmePreferenceConstants.T_MEM_ARB, EnvironmentConstants.TREE_MEMORY_SIZE_DEFAULT_VALUE);
		ps.setDefault(SmePreferenceConstants.CHAIN_MEM_ARB, EnvironmentConstants.STRING_MEMORY_SIZE_DEFAULT_VALUE);
	}
	
	/**
	 * Initializes the HMI with preference values.
	 */
	private void loadPreferences() {
		libraryPath.load();
		typeTableSize.load();
		treeMemorySize.load();
		stringMemorySize.load();
	}
	
	/**
	 * Stores the HMI values into the preference store.
	 */
	private void storePreferences() {
		libraryPath.store();
		typeTableSize.store();
		treeMemorySize.store();
		stringMemorySize.store();
	}
	
	/**
	 * Initializes the fields with default preference values.
	 */
	private void loadDefaultPreferences() {
		libraryPath.loadDefault();
		typeTableSize.loadDefault();
		treeMemorySize.loadDefault();
		stringMemorySize.loadDefault();
	}
	
	/**
	 * Creates this preference page header.
	 * 
	 * @param parent
	 *        the parent composite
	 */
	private void createHeader(Composite parent) {
		new Label(parent, SWT.WRAP).setText("General settings for the Polychrony-SME editors");
	}
	
	/**
	 * @param parent
	 *        the parent composite
	 */
	private void createFields(Composite parent) {
		Composite fieldsContainer = new Composite(parent, SWT.NONE);
		fieldsContainer.setLayout(new GridLayout());
		fieldsContainer.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		libraryPath = new PathEditor(SmePreferenceConstants.LIBRARY_PATH, "Search paths for Polychrony libraries",
			"Select directories of Polychrony libraries", fieldsContainer);
		libraryPath.setPreferenceStore(ps);
		
		typeTableSize = new IntegerFieldEditor(SmePreferenceConstants.TYPE_TABLE_SIZE,
			"Maximum number of entry in the type table (only updated after reboot)", fieldsContainer);
		typeTableSize.setPreferenceStore(ps);
		
		treeMemorySize = new IntegerFieldEditor(SmePreferenceConstants.T_MEM_ARB,
			"Memory size (in bytes) for the internal tree structure (only updated after reboot)", fieldsContainer);
		treeMemorySize.setPreferenceStore(ps);
		
		stringMemorySize = new IntegerFieldEditor(SmePreferenceConstants.CHAIN_MEM_ARB,
			"Memory size (in bytes) for the string structure (only updated after reboot)", fieldsContainer);
		stringMemorySize.setPreferenceStore(ps);
	}
	
	/**
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	// Does nothing
	}
	
	/**
	 * @see org.eclipse.jface.preference.IPreferencePage#performOk()
	 */
	public boolean performOk() {
		storePreferences();
		
		// Added to fix bug #6546
		// The Library Path is updated. Others preferences variables will only be updated after a reboot
		String value = ps.getString(SmePreferenceConstants.LIBRARY_PATH);
		PkPlugin.getServices().setEnv(EnvironmentConstants.LIBRARY_PATH, value);
		
		return super.performOk();
	}
	
	/**
	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
	 */
	protected void performDefaults() {
		loadDefaultPreferences();
		super.performDefaults();
	}
}
