/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.polychrony.traceability;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.impl.EObjectImpl;

import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * In this class is described the interface map between the native Polychrony AST and the SME AST.
 */
public class TreesInterface {
	/** String used to find the Model Declaration node in the Signal Tree */
	private final static String		MODEL_DECLARATION	= "ModelDeclarationImpl";
	
	/** Interface HashMap between the Polychrony native AST and the SME AST */
	private Map<EObjectImpl, Long>	interfaceMapSMESignal;
	
	/** List containing all the SME Nodes with their eventual error message */
	private List<SMENode>			SMENodesList;
	
	public TreesInterface() {
		interfaceMapSMESignal = new HashMap<EObjectImpl, Long>();
	}
	
	public List<SMENode> getSMENodesList() {
		return SMENodesList;
	}
	
	public Map<EObjectImpl, Long> getInterfaceMapSMESignal() {
		return interfaceMapSMESignal;
	}
	
	public void clearMap() {
		interfaceMapSMESignal.clear();
	}
	
	/**
	 * Creation of the SME nodes list with all the nodes of the interface hashmap
	 */
	public void initializeSMENodesList() {
		SMENodesList = new ArrayList<SMENode>();
		
		for (Entry<EObjectImpl, Long> entryMap : interfaceMapSMESignal.entrySet()) {
			SMENode smeNode = new SMENode();
			smeNode.setNode(entryMap.getKey());
			SMENodesList.add(smeNode);
		}
	}
	
	/**
	 * This method prints the informations key => value and value => key(s) of the interface hashmap between SME and
	 * Signal.
	 * 
	 * @param debug
	 *        set it to true if you want to display in the console the values (representing the Signal nodes in the AST)
	 *        corresponding to the EObjectImpl keys in the interface hashmap between SME and Signal.
	 * @param twosides
	 *        set it to true if you want to display in the console the EObjectImpl SME nodes corresponding to the Signal
	 *        nodes of the hashmap.
	 */
	public void printInformations(final boolean debug, final boolean twosides) {
		if (debug == true) {
			StringBuffer buffer = new StringBuffer();
			buffer.append("\n-------------------------------------------------------------------\n\n");
			buffer.append("Interface HashMap between the Polychrony native AST and the SME AST\n\n");
			buffer.append("Hashmap size : " + interfaceMapSMESignal.size() + "\n\n");
			
			for (Entry<EObjectImpl, Long> map : interfaceMapSMESignal.entrySet()) {
				EObject key = map.getKey();
				long value = map.getValue();
				buffer.append(key.toString() + "\n" + value + "\n\n");
			}
			buffer.append("\n\n");
			
			if (twosides == true) {
				List<Long> integerList = new ArrayList<Long>();
				
				for (Entry<EObjectImpl, Long> entry : interfaceMapSMESignal.entrySet()) {
					long value = entry.getValue();
					if (!integerList.contains(value)) {
						integerList.add(value);
					}
				}
				
				for (int integerCounter = 0; integerCounter < integerList.size(); integerCounter++) {
					buffer.append("VALUE n°" + (integerCounter + 1) + " : " + integerList.get(integerCounter) + "\n");
					List<EObjectImpl> listObjects = getKeys(integerList.get(integerCounter));
					for (int objectsCounter = 0; objectsCounter < listObjects.size(); objectsCounter++) {
						buffer.append("\t" + listObjects.get(objectsCounter) + "\n");
					}
					buffer.append("\n");
				}
			}
			System.out.println(buffer.toString());
		}
	}
	
	/**
	 * This methods displays a tree from its root, with the level of each node in the hierarchy.
	 * 
	 * @param node
	 *        the current node to display
	 * @param level
	 *        the level of the current node
	 * @param debug
	 *        if false, nothing is printed on the screen
	 */
	public void printTree(final long node, final int level, final boolean debug) {
		if (debug) {
			if (level == 0) {
				System.out.println("\nAbstract Syntax Tree :\n");
			}
			PKTrees API = PkPlugin.getTreeAPI();
			System.out.print("LEVEL " + level + " : ");
			for (int i = 0; i < level; i++) {
				System.out.print("......");
			}
			if (node == PKTrees.ERROR_NODE) {
				System.out.println("(error)");
			}
			if (node == API.getNilTree()) {
				System.out.println("(nil)");
			}
			else {
				System.out.print("(node : " + node + ", arity: " + API.treeArity(node) + ")");
				if (getKey(node) != null) // if there is a SME object corresponding to the node
				{
					System.out.println(" ==> " + getKey(node));
				}
				else {
					System.out.println();
				}
				if (API.oper(node) >= SignalOperators.opunary) {
					for (int i = 1; i <= API.treeArity(node); i++) {
						long child = API.child(i, node);
						printTree(child, level + 1, debug);
					}
				}
				else {
					System.out.print("LEVEL " + (level + 1) + " : ");
					for (int i = 0; i < level + 1; i++) {
						System.out.print("......");
					}
					System.out.println("(node : " + API.intOf(node) + ")");
				}
			}
		}
	}
	
	/**
	 * This method gets the node in the Signal tree corresponding to the given node in the error tree.
	 * 
	 * @param errorNode
	 *        the node in the error tree
	 * @param tree
	 *        the Signal tree
	 * @param result
	 *        the variable that will contain the result
	 * @return the corresponding node in the Signal Tree
	 */
	private long getSignalNode(final long errorNode, final long tree, long result) {
		PKTrees API = PkPlugin.getTreeAPI();
		if (tree == errorNode) {
			result = tree;
		}
		else {
			if (API.oper(tree) >= SignalOperators.opunary) {
				for (int i = 1; i <= API.treeArity(tree); i++) {
					long child = API.child(i, tree);
					result = getSignalNode(errorNode, child, result);
				}
			}
		}
		return result;
	}
	
	/**
	 * This methods prints the errors on the SME diagram
	 * 
	 * @param tree
	 *        the AST representing the diagram
	 * @param debug
	 *        prints the debug information if true
	 */
	public void printErrorsOnDiagram(final long tree, final boolean debug) {
		PKTrees API = PkPlugin.getTreeAPI();
		
		/*
		 * The error tree is given by the native getAnnotations method
		 */
		long errorTree = PkPlugin.getCommentAPI().getAnnotations(tree, API.getNilTree());
		if (errorTree != API.getNilTree()) {
			/*
			 * For each child of the error tree, the intOf method gets the integer value corresponding to the node
			 */
			for (int i = 1; i <= API.treeArity(errorTree); i++) {
				long errorTreeChild = API.child(i, errorTree);
				long intOfErrorTreeChild = API.intOf(errorTreeChild);
				
				/*
				 * The annotations tree is given by the getAnnotation method, from the integer value of each child of
				 * the error tree. The annotations tree is a tree of strings representing error or warning messages.
				 */
				long annotationsTree = PkPlugin.getCommentAPI().getAnnotation(intOfErrorTreeChild);
				if (debug) {
					System.out.println("\n-----------------");
					System.out.println("CORRESPONDENCE " + i + ":");
					System.out.println("-----------------\n");
					System.out.println("Node address : " + intOfErrorTreeChild + "\n");
				}
				
				/*
				 * The getKey method gets the nodes of the Signal tree in the SME graphical world.
				 */
				EObjectImpl smeObject = getKey(intOfErrorTreeChild);
				
				if (debug) {
					if (smeObject == null) {
						System.out.println("No correspondence in the SME nodes table\n");
					}
					else {
						System.out.println("Associated SME node :\n\t" + smeObject + "\n");
					}
				}
				
				/*
				 * For each child of the annotation tree, the message is given by the stringOf method, and the add
				 * method of the PolychronyDiagnostic class allows us to represent this error on the SME diagram.
				 */
				for (int j = 1; j <= API.treeArity(annotationsTree); j++) {
					long annotationsTreeChild = API.child(j, annotationsTree);
					String annotation = API.stringOf(annotationsTreeChild);
					if (debug) {
						System.out.println("Error / Warning message : " + annotation + "\n");
					}
					if (smeObject != null && !annotation.matches("(.)*cf no-((\\d)+)")) {
						annotation = annotation.replaceAll(" \\(ref-(.)*\\)", "");
						
						for (SMENode smeNode : SMENodesList) {
							if (smeNode.getNode().equals(smeObject)) {
								smeNode.setError(true);
								smeNode.setErrorMessage(annotation);
							}
						}
						
						PolychronyDiagnostic.add(Diagnostic.ERROR, annotation, smeObject);
						
						/*
						 * We get the node corresponding to the error node
						 */
						long node = getSignalNode(intOfErrorTreeChild, tree, 0);
						
						boolean recherche = true;
						
						/*
						 * In this part of the method, we are looking for the model declaration node in the tree
						 */
						while (recherche) {
							node = API.getFather(node);
							for (int nodesList = 1; nodesList <= API.treeArity(node); nodesList++) {
								if (((getKey(API.child(nodesList, node)) != null) && (getKey(API.child(nodesList, node))
										.toString().contains(MODEL_DECLARATION)))
										|| !API.isNilTree(node)) {
									recherche = false;
									node = API.child(nodesList, node);
								}
							}
						}
						
						/*
						 * We only want to display an error on the diagram when there is a sub-diagram, and we don't
						 * want to display an error on the main model
						 */
						if (API.getFather(node) != tree) {
							PolychronyDiagnostic.add(Diagnostic.ERROR, annotation, getKey(node));
							for (SMENode smeNode : SMENodesList) {
								if (smeNode.getNode().equals(getKey(node))) {
									smeNode.setError(true);
									smeNode.setErrorMessage(annotation);
								}
							}
						}
					}
				}
			}
		}
	}
	
	/**
	 * This method returns the key corresponding to a value in the map (In the case where only one key is corresponding
	 * to a value)
	 * 
	 * @param value
	 *        the value whose corresponding key we want
	 * @return the corresponding key
	 */
	public EObjectImpl getKey(final long value) {
		EObjectImpl key = null;
		for (Entry<EObjectImpl, Long> entryMap : interfaceMapSMESignal.entrySet()) {
			EObjectImpl currentKey = entryMap.getKey();
			long currentValue = entryMap.getValue();
			if (currentValue == value) {
				key = currentKey;
			}
		}
		return key;
	}
	
	/**
	 * This method returns a list containing the keys corresponding to a value in the map (In the case where there may
	 * be more than one key corresponding to a value)
	 * 
	 * @param value
	 *        the value whose corresponding keys we want
	 * @return the list of the corresponding keys
	 */
	private List<EObjectImpl> getKeys(final long value) {
		List<EObjectImpl> keysList = new ArrayList<EObjectImpl>();
		
		for (Entry<EObjectImpl, Long> entryMap : interfaceMapSMESignal.entrySet()) {
			EObjectImpl currentKey = entryMap.getKey();
			long currentValue = entryMap.getValue();
			
			if (currentValue == value) {
				keysList.add(currentKey);
			}
		}
		return keysList;
	}
}
