/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.polychrony.utils;

import java.io.IOException;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IOConsole;
import org.eclipse.ui.console.IOConsoleOutputStream;

public class PolychronyConsole {
	private final static PolychronyConsole	INSTANCE		= new PolychronyConsole();
	
	/** The IOConsole instance */
	private IOConsole						console			= null;
	
	private IOConsoleOutputStream			outputStream	= null;
	
	/**
	 * This basic constructor instantiates an IOConsole object, add the console instance to the eclipse console
	 * registry, and display it.
	 * 
	 * @param name
	 */
	private PolychronyConsole() {
		console = new IOConsole("Polychrony console", null);
		ConsolePlugin.getDefault().getConsoleManager().addConsoles(new IConsole[] { console });
		ConsolePlugin.getDefault().getConsoleManager().showConsoleView(console);
		console.activate();
	}
	
	/**
	 * This is a lazy initialization.
	 * 
	 * @return
	 */
	private IOConsoleOutputStream getOutputStream() {
		if (outputStream == null)
			outputStream = console.newOutputStream();
		return outputStream;
	}
	
	/**
	 * @param color
	 * @param message
	 */
	public void print(Color color, String message) {
		if (message == null)
			return;
		if (color != null)
			changeColor(color);
		
		IOConsoleOutputStream stream = getOutputStream();
		try {
			stream.write(message);
		}
		catch (IOException exception) {
			exception.printStackTrace();
		}
	}
	
	public static void printNewLine() {
		INSTANCE.print(null, "\n");
	}
	
	/**
	 * @param message
	 */
	public static void printInfo(String message) {
		INSTANCE.print(new Color(Display.getCurrent(), 0, 0, 0), message);
	}
	
	/**
	 * @param message
	 */
	public static void printWarning(String message) {
		INSTANCE.print(new Color(Display.getCurrent(), 255, 170, 0), message);
	}
	
	/**
	 * @param message
	 */
	public static void printError(String message) {
		INSTANCE.print(new Color(Display.getCurrent(), 255, 0, 0), message);
	}
	
	/**
	 * Change the color for the following messages (this is because a simple change of current stream color, change the
	 * color for all messages, even previous ones ...)
	 * 
	 * @param c
	 */
	private void changeColor(Color c) {
		Color previousColor = getOutputStream().getColor();
		if (!c.equals(previousColor)) {
			// need to change to another stream for the new color
			outputStream = null; // resets the stream
			getOutputStream().setColor(c);
		}
	}
	
	public void clear() {
		console.clearConsole();
	}
	
	public void dispose() {
		try {
			outputStream.close();
		}
		catch (IOException exception) {
			exception.printStackTrace();
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	public void finalize() throws Throwable {
		super.finalize();
		if ((outputStream != null) && !outputStream.isClosed())
			outputStream.close();
	}
}
