/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.polychrony.utils;

import org.eclipse.emf.common.util.BasicDiagnostic;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.ecore.EObject;

/**
 * Class used to accumulate the diagnostic during a call of the native part. To use it, clear it before each time you
 * apply the native part functionality to a new SME file.
 */
public class PolychronyDiagnostic {
	/** Used as accumulator of Diagnostic */
	private static BasicDiagnostic	currentDiagnostic;
	
	public static synchronized void add(int severity, String message, EObject obj) {
		getCurrentDiagnostic().merge(
			new BasicDiagnostic(severity, PolychronyMarkerHelper.ID_MARKER, 0, message, new Object[] { obj }));
	}
	
	public static synchronized void clear() {
		currentDiagnostic = null;
	}
	
	/**
	 * @return
	 */
	public static synchronized BasicDiagnostic getCurrentDiagnostic() {
		if (currentDiagnostic == null)
			currentDiagnostic = new BasicDiagnostic();
		return currentDiagnostic;
	}
	
	/**
	 * Iterates on all the Diagnostics contained by the BasicDiagnostic and removes those which appears more than once.
	 * 
	 * @return a Diagnostic with only unique Diagnostics
	 */
	public static synchronized Diagnostic getUniqueDiagnostics() {
		BasicDiagnostic result = new BasicDiagnostic();
		
		// Iterates on all the diagnostics found
		for (Diagnostic currentDiag : getCurrentDiagnostic().getChildren()) {
			boolean alreadyExist = false;
			
			// checks if the Diagnostic has not been already added
			for (Diagnostic uniqueDiag : result.getChildren()) {
				// checks the equality of the two Diagnostics
				if (currentDiag.getSeverity() == uniqueDiag.getSeverity()
						&& currentDiag.getCode() == uniqueDiag.getCode()
						&& currentDiag.getSource().equals(uniqueDiag.getSource())
						&& currentDiag.getMessage().equals(uniqueDiag.getMessage())
						&& currentDiag.getData().containsAll(uniqueDiag.getData())) {
					alreadyExist = true;
					break;
				}
			}
			if (!alreadyExist)
				result.add(currentDiag);
		}
		return result;
	}
}
