/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.util;

import java.util.Comparator;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.ECollections;
import org.eclipse.emf.common.util.EList;

import fr.irisa.espresso.sme.CartesianProduct;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.Iterate;
import fr.irisa.espresso.sme.ModelDeclaration;
import fr.irisa.espresso.sme.MultipleInputsOperator;
import fr.irisa.espresso.sme.OutputPort;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalOperator;
import fr.irisa.espresso.sme.SingleInputOperator;
import fr.irisa.espresso.sme.SmeFactory;
import fr.irisa.espresso.sme.Transition;
import fr.irisa.espresso.sme.TwoInputsOperator;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;

/**
 * Utility functions
 */
public class Utils {
	/*
	 * @generated NOT
	 */
	private static boolean	btrace	= false;
	
	/**
	 * This methods displays on the standard output the structure of an AST
	 * 
	 * @param child2
	 *        the AST to analyze
	 * @param level
	 *        the hierarchic level (display a tab for each new level)
	 */
	public static void analyzeTree(long child2, int level) {
		PKTrees API = PkPlugin.getTreeAPI();
		
		for (int i = 0; i < level; i++)
			System.out.print("\t");
		if (child2 == PKTrees.ERROR_NODE || child2 == API.getNilTree())
			System.out.println("(nil)");
		else {
			int op = API.oper(child2);
			int arity = API.treeArity(child2);
			System.out.println("(op : " + op + ", arity: " + arity + ")");
			if (op >= SignalOperators.opunary) {
				for (int i = 1; i <= arity; i++) {
					long child = API.child(i, child2);
					analyzeTree(child, level + 1);
				}
			}
		}
	}
	
	/**
	 * @param eList
	 *        a list of Signal Elements
	 * @param l
	 *        a node corresponding to a list of type ldecl (assumed).
	 * @return a node corresponding to the declarations of each element of <code>eList</code> added to the end of
	 *         <code>listTree</code>.
	 */
	public static long makeDeclarationList(EList<? extends SignalElement> eList, long l) {
		if (eList != null && !eList.isEmpty())
			for (SignalElement se : eList)
				l = se.getDeclarationAST(l);
		return l;
	}
	
	/**
	 * @param eList
	 *        a list of Signal Elements
	 * @param body
	 *        a node corresponding to a list of type lfeed (assumed).
	 * @return a node corresponding to the equations of each element of <code>eList</code> added to the end of
	 *         <code>listTree</code>.
	 */
	public static long makeEquationsList(EList<? extends SignalElement> eList, long body) {
		if (eList != null && !eList.isEmpty()) {
			for (SignalElement se : eList)
				body = se.buildEquationsAST(body);
		}
		return body;
	}
	
	/**
	 * @param id
	 *        the identifier
	 * @param pragmaList
	 *        assumed to be a node representing the list of pragmas
	 * @param withId
	 *        use the identifier in the pragma declaration
	 * @return the list of pragmas modified by adding the AST of the comment pragma of the identifier if any.
	 */
	public static long getCommentAST(Identifier id, long pragmaList) {
		String comment = id.getComment();
		if (comment != null && "".equals(comment) == false) {
			PKTrees API = PkPlugin.getTreeAPI();
			try {
				// It is an identifier so the set is null
				long idNode;
				if (id instanceof ModelDeclaration)
					idNode = API.makeNullary(SignalOperators.opnil);
				else
					idNode = API.makeList(SignalOperators.oplopra, id.getAST(null));
				if (idNode != PKTrees.ERROR_NODE) {
					pragmaList = API.post(pragmaList, API.makeTernary(SignalOperators.opprag, API.atomString(
						SignalOperators.opnom, "Comment"), idNode, API.makePragmaEnonce(comment)));
				}
			}
			catch (CyclicDefinitionException cde) {
				// Cannot be thrown normally
			}
		}
		return pragmaList;
	}
	
	/**
	 * Initialize the operator by adding InputPorts. For operators of type :
	 * <ul>
	 * <li>SingleInputOperator, if the operator has no Input Port, it adds one.</li>
	 * <li>TwoInputsOperator, it adds an operator for the first and second operand (if not initialized yet).</li>
	 * <li>MultipleInputsOperator, it adds Input Ports to have at least two ports.</li>
	 * </ul>
	 * 
	 * @param operator
	 *        the operator to initialize
	 * @return the modified operator
	 */
	public static SignalOperator createPorts(SignalOperator operator) {
		if (operator instanceof MultipleInputsOperator) {
			int max = 2;
			if (operator instanceof CartesianProduct) {
				CartesianProduct car = (CartesianProduct) operator;
				max = Math.max(max, Math.max(car.getOuts().size(), car.getIns().size()));
				for (int i = car.getOuts().size(); i < max; i++) {
					OutputPort out = SmeFactory.eINSTANCE.createOutputPort();
					car.getOuts().add(out);
					out.setOperatorParent(car);
				}
			}
			
			MultipleInputsOperator op = (MultipleInputsOperator) operator;
			for (int i = op.getIns().size(); i < max; i++) {
				InputPort in = SmeFactory.eINSTANCE.createInputPort();
				op.getIns().add(in);
				in.setOperatorParent(op);
			}
			
		}
		else if (operator instanceof SingleInputOperator) {
			SingleInputOperator op = (SingleInputOperator) operator;
			if (op.getIn() == null) {
				InputPort in = SmeFactory.eINSTANCE.createInputPort();
				op.setIn(in);
				in.setOperatorParent(op);
			}
		}
		else if (operator instanceof TwoInputsOperator) {
			TwoInputsOperator op = (TwoInputsOperator) operator;
			if (op.getFirstOperand() == null) {
				InputPort in = SmeFactory.eINSTANCE.createInputPort();
				op.setFirstOperand(in);
				in.setOperatorParent(op);
			}
			if (op.getSecondOperand() == null) {
				InputPort in = SmeFactory.eINSTANCE.createInputPort();
				op.setSecondOperand(in);
				in.setOperatorParent(op);
			}
		}
		else if (operator instanceof Iterate) {
			Iterate op = (Iterate) operator;
			if (op.getIns().size() == 0) {
				InputPort in = SmeFactory.eINSTANCE.createInputPort();
				op.getIns().add(in);
				in.setOperatorParent(op);
			}
		}
		return operator;
	}
	
	/**
	 * This method returns an ordered list of all transitions given as argument. This list is ordered according to the
	 * priority of the transitions.
	 * 
	 * @param transitions
	 *        the list of transitions to order
	 * @return a list of transitions ordered
	 */
	public static EList<Transition> orderTransitions(EList<? extends Transition> transitions) {
		EList<Transition> orderedList = new BasicEList<Transition>(transitions);
		ECollections.sort(orderedList, new Comparator<Transition>() {
			public int compare(Transition tr1, Transition tr2) {
				return (tr1.getPriority() - tr2.getPriority());
			}
		});
		return orderedList;
	}
	
	/**
	 * This method returns the string to parse to obtain the source or destination of a connector
	 * 
	 * @param tmp
	 *        the String to return
	 * @param SrcOrDst
	 *        srcField or dstField
	 * @return the string to parse
	 * @generated NOT
	 */
	public static String getSignalString(String tmp, String SrcOrDst) {
		SrcOrDst = SrcOrDst.replaceAll(" ", "");
		
		if (SrcOrDst.startsWith("["))
			// the source/destination field indicate the index of an array element
			tmp += SrcOrDst;
		else if (SrcOrDst.startsWith("."))
			// the source/destination field indicate the name of a subfield and we do not need to add a '.'
			tmp += SrcOrDst;
		else
			// the source/destination field indicate the name of a subfield and we need to add a '.'
			tmp += "." + SrcOrDst;
		return tmp;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#buildEquationsAST(int)
	 * @generated NOT
	 */
	public static void TRACE(String s) {
		if (btrace)
			System.out.println(s);
	}
	
}
