/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.texteditor;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IWordDetector;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.NumberRule;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WordRule;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;

/**
 * This class defines the Signal language Scanner for the Signal textual editor
 */
public class SignalEditorScanner extends RuleBasedScanner {
	/** Colors for text elements */
	private static final Color		COLOR_KEYWORD;
	private static final Color		COLOR_COMMENT;
	private static final Color		COLOR_CONSTANT;
	private static final Color		COLOR_OTHER;
	
	static {
		Display d = PlatformUI.getWorkbench().getDisplay();
		COLOR_KEYWORD = new Color(d, 127, 0, 85);
		COLOR_COMMENT = new Color(d, 0, 128, 0);
		COLOR_CONSTANT = new Color(d, 0, 0, 255);
		COLOR_OTHER = new Color(d, 0, 0, 0);
	}
	
	/** List of Signal language keywords */
	private static final String[]	KEYWORDS	= new String[] { "action", "after", "and", "array", "assert",
			"boolean", "bundle", "case", "cell", "char", "complex", "constant", "count", "dcomplex", "default",
			"defaultvalue", "deterministic", "dreal", "else", "end", "enum", "event", "external", "false", "from",
			"function", "if", "in", "init", "integer", "iterate", "label", "long", "module", "modulo", "next", "node",
			"not", "of", "operator", "or", "pragmas", "private", "process", "real", "ref", "safe", "shared", "short",
			"spec", "statevar", "step", "string", "struct", "then", "to", "tr", "true", "type", "unsafe", "use", "var",
			"when", "where", "window", "with", "xor" };
	
	/**
	 * Constructor of Signal editor Scanner
	 */
	public SignalEditorScanner() {
		Token keyword = new Token(new TextAttribute(COLOR_KEYWORD, null, SWT.BOLD));
		Token comment = new Token(new TextAttribute(COLOR_COMMENT, null, SWT.ITALIC));
		Token constant = new Token(new TextAttribute(COLOR_CONSTANT, null, SWT.BOLD));
		Token other = new Token(new TextAttribute(COLOR_OTHER));
		
		WordRule basicRule = new WordRule(new IWordDetector() {
			public boolean isWordPart(char c) {
				return Character.isJavaIdentifierPart(c);
			}
			public boolean isWordStart(char c) {
				return Character.isJavaIdentifierStart(c);
			}
		}, other);
		
		for (String k : KEYWORDS)
			basicRule.addWord(k, keyword);
		
		IRule numberRule = new NumberRule(constant);
		IRule charRule = new SingleLineRule("'", "'", constant, '\\', true);
		IRule stringRule = new MultiLineRule("\"", "\"", constant, '\\', true);
		
		IRule commentRule = new MultiLineRule("%", "%", comment, (char) 0, true);
		
		setRules(new IRule[] { basicRule, commentRule, numberRule, charRule, stringRule });
	}
}
